#include "program_version.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <hardware/btn-hardware.h>
#include <terminal/btn-terminal.h>
#include <terminal/btn-cmds.h>
#include <terminal/nut-cmds.h>
#include <led/btn-led.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <cc/bmac.h>
#include <cc/ccc.h>
#include <cc/cc1000.h> // for rf_power
#include <wsnlab/wsnlab.h>

// ----------------------------------------------------------------------------
// #define constants
// ----------------------------------------------------------------------------

#define PROJECT_NAME "template"  // TODO: change this to your project name
#define UART_SPEED   57600

// ----------------------------------------------------------------------------
// global variables
// ----------------------------------------------------------------------------

// ...

 
// ----------------------------------------------------------------------------
// helper functions
// ----------------------------------------------------------------------------

/**
 * Initializes the BTnode.
 */
static void btnode_init(void) 
{    
    /* initialize the btnode hardware */
    btn_hardware_init();
    btn_led_init(0);

    char prompt[TERMINAL_MAX_PROMPT_LENGTH];
    u_short mac_address = get_MAC_address_from_EEPROM();
    
    if ( mac_address == 0 )
    {
    	// no printf() here -- terminal not inited yet!
    	mac_address = 0xEEEE;    
    }
    
    //initialise random number generator
    srand( mac_address );
        
    sprintf( prompt, "[%s@%02X:%02X]$", PROJECT_NAME, mac_address >> 8, 
    	mac_address & 0xFF );
    
    term_init(UART_SPEED, prompt);     
    
    int res = bmac_init(mac_address);
    if (res != 0)
    {
    	printf( "bmac_init() failed\n" );
    }
    bmac_enable_led( 1 );
    res = ccc_init(&bmac_interface);
    if (res != 0)
    {
    	printf( "ccc_init() failed\n" );
    }
}

// ----------------------------------------------------------------------------
// terminal commands
// ----------------------------------------------------------------------------

/**
 * Registers all supported commands.
 */
static void register_cmds(void)
{
	nut_cmds_register_cmds();
    btn_cmds_register_cmds();
    wsnlab_cmds_register_cmds();

	// register additional commands here
    //btn_terminal_register_cmd("test", test_cmd);
}
  
int main(void)
{
	btnode_init();
    
    DEBUGB("\n# ----------------------------------------");
    DEBUGB("\n# " PROJECT_NAME " version " PROGRAM_VERSION "\n");
    DEBUGB("# ----------------------------------------\n");
    NutSleep(200);
 
    // register cmds
	register_cmds();
	
    // start terminal
    btn_terminal_run(BTN_TERMINAL_NOFORK, 1024);
    
    // we should never reach this point
    return 1;
}
